# -*- coding: utf-8 -*-
from __future__ import unicode_literals

try:
    from django.contrib.sites.shortcuts import get_current_site
except ImportError:
    from django.contrib.sites.models import get_current_site
from django.core.exceptions import ObjectDoesNotExist
from django.apps import apps
from django.utils.encoding import python_2_unicode_compatible

from cmsplugin_cascade.models import InlineCascadeElement, SortableInlineCascadeElement


@python_2_unicode_compatible
class ImagePropertyMixin(object):
    """
    A mixin class to convert a CascadeElement into a proxy model for rendering an image element.
    """
    def __str__(self):
        try:
            return self.plugin_class.get_identifier(self)
        except AttributeError:
            return str(self.image)

    @property
    def image(self):
        if not hasattr(self, '_image_model'):
            try:
                Model = apps.get_model(*self.glossary['image']['model'].split('.'))
                self._image_model = Model.objects.get(pk=self.glossary['image']['pk'])
            except (KeyError, ObjectDoesNotExist):
                self._image_model = None
        return self._image_model


class WithInlineElementsMixin(object):
    """
    Plugins wishing to allow child elements as inlines, shall inherit from this
    mixin class, in order to override the serialize and deserialize methods.
    """
    @classmethod
    def get_data_representation(cls, instance):
        inlines = [ie.glossary for ie in instance.inline_elements.all()]
        return {'glossary': instance.glossary, 'inlines': inlines}

    @classmethod
    def add_inline_elements(cls, instance, inlines):
        for inline_glossary in inlines:
            InlineCascadeElement.objects.create(
                cascade_element=instance, glossary=inline_glossary)


class WithSortableInlineElementsMixin(object):
    """
    Plugins wishing to allow child elements as sortable inlines, shall inherit from this
    mixin class, in order to override the serialize and deserialize methods.
    """
    @classmethod
    def get_data_representation(cls, instance):
        inlines = [ie.glossary for ie in instance.sortinline_elements.all()]
        return {'glossary': instance.glossary, 'inlines': inlines}

    @classmethod
    def add_inline_elements(cls, instance, inlines):
        for order, inline_glossary in enumerate(inlines, 1):
            SortableInlineCascadeElement.objects.create(
                cascade_element=instance, glossary=inline_glossary, order=order)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os
from bs4 import BeautifulSoup
import json

from django.contrib.auth import get_user_model
from django.test.client import Client
from django.core.urlresolvers import reverse, resolve
from django.core.files.uploadedfile import SimpleUploadedFile

from filer.admin.clipboardadmin import ajax_upload

from cms.api import add_plugin
from cms.utils.plugins import build_plugin_tree

from cmsplugin_cascade.models import IconFont
from cmsplugin_cascade.settings import CMSPLUGIN_CASCADE
from cmsplugin_cascade.bootstrap3.container import BootstrapContainerPlugin
from cmsplugin_cascade.icon.cms_plugins import FramedIconPlugin
from .test_base import CascadeTestCase

BS3_BREAKPOINT_KEYS = list(tp[0] for tp in CMSPLUGIN_CASCADE['bootstrap3']['breakpoints'])


class IconFontTestCase(CascadeTestCase):
    client = Client()

    def setUp(self):
        super(IconFontTestCase, self).setUp()
        UserModel = get_user_model()
        self.admin_user = UserModel.objects.get(username='admin')

    def test_add_font(self):
        with self.login_user_context(self.admin_user):
            # upload the zipfile into the filer's clipboard
            filename = os.path.join(os.path.dirname(__file__), 'assets/fontello-b504201f.zip')
            with open(filename, 'rb') as zipfile:
                uploaded_file = SimpleUploadedFile('fontello-b504201f.zip', zipfile.read(), content_type='application/zip')
            request = self.get_request(reverse('admin:filer-ajax_upload'))
            self.assertTrue(request.user.is_staff, "User is not a staff user")
            request.FILES.update(file=uploaded_file)
            response = ajax_upload(request)
            self.assertEqual(response.status_code, 200)
            content = json.loads(response.content.decode('utf-8'))
            self.assertDictContainsSubset({'label': 'fontello-b504201f.zip'}, content)

            # save the form and submit the remaining fields
            add_iconfont_url = reverse('admin:cmsplugin_cascade_iconfont_add')
            data = {
                'identifier': "Fontellico",
                'zip_file': content['file_id'],
                '_continue': "Save and continue editing",
            }
            response = self.client.post(add_iconfont_url, data)
            self.assertEqual(response.status_code, 302)
            resolver_match = resolve(response.url)
            self.assertEqual(resolver_match.url_name, 'cmsplugin_cascade_iconfont_change')

            # check the content of the uploaded file
            icon_font = IconFont.objects.get(pk=resolver_match.args[0])
            self.assertEqual(icon_font.identifier, "Fontellico")
            self.assertEqual(icon_font.config_data['name'], 'fontelico')
            self.assertEqual(len(icon_font.config_data['glyphs']), 34)

            # check if the uploaded fonts are rendered inside Preview Icons
            response = self.client.get(response.url)
            self.assertEqual(response.status_code, 200)
            soup = BeautifulSoup(response.content, 'lxml')
            preview_iconfont = soup.find('div', class_="preview-iconfont")
            icon_items = preview_iconfont.ul.find_all('li')
            self.assertEqual(len(icon_items), 34)
            self.assertListEqual(icon_items[0].i.attrs['class'], ['icon-emo-happy'])
            self.assertListEqual(icon_items[33].i.attrs['class'], ['icon-marquee'])

            # create container
            container_model = add_plugin(self.placeholder, BootstrapContainerPlugin, 'en',
                                         glossary={'breakpoints': BS3_BREAKPOINT_KEYS})
            container_plugin = container_model.get_plugin_class_instance()
            self.assertIsInstance(container_plugin, BootstrapContainerPlugin)

            # add icon as child to this container
            glossary = {"font_size": "10em", "color": "#88258e", "background_color": ["on","#c8ffcd"],
                        "symbol": "emo-wink", "icon_font": icon_font.pk,
                        "border_radius":"50%","border":["","solid","#000000"]}
            icon_model = add_plugin(self.placeholder, FramedIconPlugin, 'en', target=container_model,
                                    glossary=glossary)
            icon_plugin = icon_model.get_plugin_class_instance()
            self.assertIsInstance(icon_plugin, FramedIconPlugin)

            # render the plugins
            plugin_list = [container_model, icon_model]
            build_plugin_tree(plugin_list)
            html = self.get_html(container_model, self.get_request_context())
            soup = BeautifulSoup(html, 'lxml')

            # look for the icon symbol
            style = soup.find('span', class_='icon-emo-wink').attrs['style'].split(';')
            self.assertIn('color:#88258e', style)
            self.assertIn('display:inline-block', style)

            # look for the CSS file
            response = self.client.get(container_model.placeholder.page.get_absolute_url() + '?edit')
            self.assertEqual(response.status_code, 200)
            soup = BeautifulSoup(response.content, 'lxml')
            links = soup.head.find_all('link')
            for link in links:
                if link.attrs['href'].endswith('fontelico.css'):
                    break
            else:
                self.fail("No CSS file with font definition found")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from django.template.context import Context

from cms.api import create_page
from cms.test_utils.testcases import CMSTestCase

from djangocms_helper.base_test import BaseTestCase


class CascadeTestCase(CMSTestCase, BaseTestCase):
    home_page = None

    def setUp(self):
        self.home_page = create_page(title='HOME', template='testing.html', language='en')
        self.placeholder = self.home_page.placeholders.get(slot='Main Content')

        self.request = self.get_request(self.home_page, 'en')
        self.admin_site = admin.sites.AdminSite()

    def get_request_context(self):
        context = {}
        context['request'] = self.request
        context['user'] = self.request.user
        try:
            # >= Django CMS v3.4.x
            context['cms_content_renderer'] = self.get_content_renderer(request=self.request)
        except AttributeError:
            # < Django CMS v3.4.x
            pass
        return Context(context)

    def get_html(self, model_instance, context):
        try:
            # >= Django CMS v3.4.x
            return context['cms_content_renderer'].render_plugin(model_instance, context)
        except KeyError:
            # < Django CMS v3.4.x
            return model_instance.render_plugin(context)



try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from django.test import TestCase
from django.core.management import call_command


class MissingMigrationTest(TestCase):

    def test_for_missing_migrations(self):
        out = StringIO()

        call_command('makemigrations', '--dry-run', 'cmsplugin_cascade',
                     verbocity=3, interactive=False, stdout=out)
        self.assertEquals(out.getvalue(), "No changes detected in app 'cmsplugin_cascade'\n")
